@@ -6,6 +6,12 @@ use warnings;
 our $es;
 my $r;
 
+# version 0.16 is sometimes failing when creating a new index during
+# bulk indexing.
+
+$es->create_index( index => 'es_test_3' );
+wait_for_es();
+
 ok $es->reindex(
     source => $es->scrolled_search(
         index       => 'es_test_1',
@@ -36,4 +42,4 @@ is $r->{hits}{total},      6,    ' - skip docs';
 is $first->{_source}{num}, 1011, ' - transform';
 is $first->{_version}, 1, ' - version';
 
-1
+1;
